/* GNU Mailutils -- a suite of utilities for electronic mail
   Copyright (C) 1999-2025 Free Software Foundation, Inc.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General
   Public License along with this library.  If not, see 
   <http://www.gnu.org/licenses/>. */

#ifndef _MAILUTILS_SYS_MONITOR_H
# define _MAILUTILS_SYS_MONITOR_H

# ifdef HAVE_CONFIG_H
#  include <config.h>
# endif

# include <sys/types.h>
# include <mailutils/monitor.h>

# ifdef __cplusplus
extern "C" {
# endif
/* FIXME: any protos? */
# ifdef __cplusplus
}
# endif

#endif /* _MONITOR0_H */
